@@ -59,10 +59,16 @@ void gpx_free_tb(tb_t *tb) {
   free(tb);
 }
 
+void gpx_free_attribute(attribute_t *att) {
+  if(att->name) xmlFree(att->name);
+  free(att);
+}
+
 void gpx_free_cache(cache_t *cache) {
-  log_t *log = cache->log;
-  wpt_t *wpt = cache->wpt;
-  tb_t  *tb  = cache->tb;
+  log_t        *log = cache->log;
+  wpt_t        *wpt = cache->wpt;
+  tb_t         *tb  = cache->tb;
+  attribute_t  *att = cache->attribute;
 
   if(cache->id)                xmlFree(cache->id);
   if(cache->name)              xmlFree(cache->name);
@@ -81,6 +87,9 @@ void gpx_free_cache(cache_t *cache) {
   /* free all tbs */
   while(tb) { tb_t *next = tb->next; gpx_free_tb(tb); tb = next; }
 
+  /* free all attributes */
+  while(att) { attribute_t *next = att->next; gpx_free_attribute(att); att = next; }
+
   if(cache->notes)             notes_free(cache->notes);
 
   free(cache);
@@ -332,6 +341,16 @@ static float xml_get_prop_float(xmlTextReaderPtr reader, char *name) {
   return ret;
 }
 
+static int xml_get_prop_int(xmlTextReaderPtr reader, char *name) {
+  int ret = -1;
+  char *prop;
+  if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST name))) {
+    ret = atoi(prop);
+    xmlFree(prop);
+  }
+  return ret;
+}
+
 static unsigned int xml_get_prop_id(xmlTextReaderPtr reader) {
   unsigned int ret = 0;
   char *prop;
@@ -661,6 +680,55 @@ static tb_t *process_gpx_wpt_gc_tbs(xmlTextReaderPtr reader) {
   return NULL;
 }
 
+static void process_gpx_wpt_gc_atts(xmlTextReaderPtr reader, 
+				    cache_t *cache) {
+  if(xmlTextReaderIsEmptyElement(reader))
+    return;
+
+  attribute_t **att = &cache->attribute;
+  while(*att) att = &(*att)->next;
+
+  /* process all sub-nodes */
+  int depth = xmlTextReaderDepth(reader);
+  int ret = xmlTextReaderRead(reader);
+  while(ret == 1) {
+    
+    switch(xmlTextReaderNodeType(reader)) {
+    case XML_READER_TYPE_ELEMENT:
+      g_assert(xmlTextReaderDepth(reader) == depth+1);
+      char *name = (char*)xmlTextReaderConstName(reader);
+      if(strrchr(name, ':')) name = strrchr(name, ':')+1;
+      if(name) {
+	if(strcasecmp(name, "attribute") == 0) {
+	  *att = g_new0(attribute_t, 1);
+	  (*att)->name = process_text(reader);
+	  (*att)->id = xml_get_prop_id(reader);
+	  if((*att)->id < 0 || (*att)->id > ATT_MAX)
+	    (*att)->id = ATT_UNKNOWN;
+	  (*att)->inc = xml_get_prop_int(reader, "inc");
+
+	  att = &(*att)->next;
+	} else {
+	  // printf("unhandled item found: gpx/wpt/cache/attributes/%s\n", name);
+	  skip_element(reader);
+	}
+      } else
+	skip_element(reader);
+      break;
+      
+    case XML_READER_TYPE_END_ELEMENT:
+      /* end element must be for the current element */
+      g_assert(xmlTextReaderDepth(reader) == depth);
+      return;
+      break;
+      
+    default:
+      break;
+    }
+    ret = xmlTextReaderRead(reader);
+  }
+}
+
 static void process_gpx_wpt_gc(xmlTextReaderPtr reader, 
 			       cache_t *cache, char *username) {
   cache->available = xml_prop_is(reader, "available", "true", TRUE);
@@ -697,6 +765,8 @@ static void process_gpx_wpt_gc(xmlTextReaderPtr reader,
 	} else if(strcasecmp(name, "container") == 0) {
 	  cache->container = xml_str_search(reader, cache_container_str, 
 					    "container", CACHE_CONT_UNKNOWN);
+	} else if(strcasecmp(name, "attributes") == 0) {
+	  process_gpx_wpt_gc_atts(reader, cache);
 	} else if((strcasecmp(name, "short_description") == 0) ||
 		  (strcasecmp(name, "summary") == 0))	{
 	  if(!cache->short_description) {
