@@ -98,7 +98,8 @@ gpointer loader_thread(gpointer data) {
 
   result = gnome_vfs_open(&handle, context->url, GNOME_VFS_OPEN_READ);
   if(result != GNOME_VFS_OK) {
-    g_print("%p: open error: %s\n", self, gnome_vfs_result_to_string(result));
+    g_print("%p: open error: %s\n(is libgnomevfs2-extra installed?)\n", 
+	    self, gnome_vfs_result_to_string(result));
 
     release_load_context(self, context);
     return NULL;
