@@ -41,6 +41,17 @@
 #include <X11/Xatom.h>
 #endif
 
+/* any defined key enables key support */
+#if (defined(MAP_KEY_FULLSCREEN) || \
+     defined(MAP_KEY_ZOOMIN) || \
+     defined(MAP_KEY_ZOOMOUT) || \
+     defined(MAP_KEY_UP) || \
+     defined(MAP_KEY_DOWN) || \
+     defined(MAP_KEY_LEFT) || \
+     defined(MAP_KEY_RIGHT))
+#include <gdk/gdkkeysyms.h>
+#endif
+
 /* default values */
 #define MAP_SOURCE  OSM_GPS_MAP_SOURCE_OPENCYCLEMAP
 #define GPS_DEFAULT_ZOOM 13
@@ -84,13 +95,64 @@ static const char *get_proxy_uri(appdata_t *appdata) {
   return NULL;
 }
 
+
+pos_t *map_gps_get_pos(map_context_t *context) {
+  static pos_t pos;
+
+  if(context->gps.set & FIX_LATLON_SET) {
+    pos.lat = context->gps.fix.latitude;
+    pos.lon = context->gps.fix.longitude;
+    return &pos;
+  }
+
+  return NULL;
+}
+
+pos_t *map_get_pos(map_context_t *context) {
+  pos_t *pos = &context->appdata->home;
+
+  if(context->appdata->active_location) {
+    int i = context->appdata->active_location-1;
+    location_t *loc = context->appdata->location;
+    while(i--) loc = loc->next;
+    pos = &loc->pos;
+  }
+
+  if(context->appdata->use_gps) 
+    pos = map_gps_get_pos(context);
+
+  return pos;
+}
+
+float map_gps_get_heading(map_context_t *context) {
+  if(context->gps.set & FIX_TRACK_SET) 
+    return context->gps.fix.track;
+
+  return NAN;
+}
+
+float map_gps_get_eph(map_context_t *context) {
+  if(context->gps.set & FIX_HERR_SET) 
+    return context->gps.fix.eph;
+
+  return NAN;
+}
+
+/* callback called by the gps layer whenever gps state changes */
+static void 
+gps_callback(gps_mask_t set, struct gps_t *fix, void *data) {
+  map_context_t *context = (map_context_t*)data;
+
+  printf("map: gps callback\n");
+
+  context->gps.set = set;
+  memcpy(&context->gps.fix, fix, sizeof(struct gps_t));
+}
+
 static void
 cb_map_gps(osd_button_t but, map_context_t *context) {
 
   if(but == OSD_GPS) {
-    printf("clicked OSD_GPS %p\n", context);
-    printf("clicked OSD_GPS a %p\n", context->appdata);
-
     pos_t *refpos = get_pos(context->appdata);
     if(refpos && !isnan(refpos->lat) && !isnan(refpos->lon)) {
       gint zoom;
@@ -126,7 +188,7 @@ static gboolean map_gps_update(gpointer data) {
 #endif
 
   /* get reference position ... */
-  pos_t *refpos = get_pos(context->appdata);
+  pos_t *refpos = map_get_pos(context);
   gboolean ok = (refpos!= NULL) && !isnan(refpos->lat) && !isnan(refpos->lon);
 
   /* ... and enable "goto" button if it's valid */
@@ -141,10 +203,10 @@ static gboolean map_gps_update(gpointer data) {
     int radius = 0;
 
     if(context->appdata->use_gps) {
-      heading = gps_get_heading(context->appdata);
+      heading = map_gps_get_heading(context);
 
       /* get error */
-      float eph = gps_get_eph(context->appdata);
+      float eph = map_gps_get_eph(context);
       if(!isnan(eph)) 
 	radius = dist2pixel(context, eph/1000, refpos->lat);
     }
@@ -825,6 +887,10 @@ static void on_window_destroy(GtkWidget *widget, map_context_t *context) {
   g_object_get(map, "zoom", &zoom, NULL);
   context->appdata->map.zoom = zoom;
 
+  gboolean dpix;
+  g_object_get(map, "double-pixel", &dpix, NULL);
+  context->appdata->map.dpix = dpix;
+
   gfloat lat, lon;
   g_object_get(map, "latitude", &lat, "longitude", &lon, NULL);
   context->appdata->map.pos.lat = lat;
@@ -840,7 +906,6 @@ static void on_window_destroy(GtkWidget *widget, map_context_t *context) {
 #endif
 
   gtk_timeout_remove(context->handler_id);
-  gps_unregister_callback(appdata, context->cb_id);
 
   if(context->caches_displayed) {
     g_free(context->caches_displayed);
@@ -887,6 +952,10 @@ static gboolean on_focus_in(GtkWidget *widget, GdkEventFocus *event,
     context->handler_id = gtk_timeout_add(1000, map_gps_update, context);
 #endif
 
+  gps_register_callback(context->appdata->gps_state, 
+			LATLON_CHANGED | HERR_CHANGED | TRACK_CHANGED,
+			gps_callback, context);
+
   map_setup(context);
   return FALSE;
 }  
@@ -909,6 +978,8 @@ static gboolean on_focus_out(GtkWidget *widget, GdkEventFocus *event,
   context->handler_id = 0;
 #endif
 
+  gps_unregister_callback(context->appdata->gps_state, gps_callback);
+
   return FALSE;
 }
 
@@ -920,10 +991,23 @@ void map_update(appdata_t *appdata) {
 #endif
 }
 
-/* callback called by the gps layer whenever gps state changes */
-static void 
-gps_callback(struct gps_state *state, gpointer data) {
-  printf("map: gps callback\n");
+static gboolean 
+on_map_window_key_press(GtkWidget *window, GdkEventKey *event, GtkWidget *map)  {
+#ifdef USE_MAEMO
+  if((event->keyval == HILDON_HARDKEY_FULLSCREEN) || 
+     (event->keyval == HILDON_HARDKEY_INCREASE) || 
+     (event->keyval == HILDON_HARDKEY_DECREASE)) 
+#else
+  if(event->keyval == GDK_F11) 
+#endif
+  {
+    gboolean return_val;
+    g_signal_emit_by_name(GTK_OBJECT(map), "key_press_event", 
+			  event, &return_val);
+    return return_val;
+  }
+  
+  return FALSE;
 }
 
 void map(appdata_t *appdata) {
@@ -978,6 +1062,7 @@ void map(appdata_t *appdata) {
 		 "auto-center",              FALSE,
 		 "record-trip-history",      FALSE, 
 		 "show-trip-history",        FALSE, 
+	         "double-pixel",             context->appdata->map.dpix,
 		 proxy?"proxy-uri":NULL,     proxy,
                  NULL);
 
@@ -1007,7 +1092,22 @@ void map(appdata_t *appdata) {
       context->appdata->map.zoom = 1;
     }
   }
-  
+
+  int key_maps[][2] = {
+    { OSM_GPS_MAP_KEY_FULLSCREEN, MAP_KEY_FULLSCREEN },
+    { OSM_GPS_MAP_KEY_ZOOMIN, MAP_KEY_ZOOMIN },
+    { OSM_GPS_MAP_KEY_ZOOMOUT, MAP_KEY_ZOOMOUT },
+    { OSM_GPS_MAP_KEY_UP, MAP_KEY_UP },
+    { OSM_GPS_MAP_KEY_DOWN, MAP_KEY_DOWN },
+    { OSM_GPS_MAP_KEY_LEFT, MAP_KEY_LEFT },
+    { OSM_GPS_MAP_KEY_RIGHT, MAP_KEY_RIGHT },
+    { OSM_GPS_MAP_KEY_MAX, 0 } };
+
+  int i; 
+  for(i=0;key_maps[i][0] != OSM_GPS_MAP_KEY_MAX;i++) 
+    osm_gps_map_set_keyboard_shortcut(OSM_GPS_MAP(context->widget), 
+				      key_maps[i][0], key_maps[i][1]);
+
   osm_gps_map_set_mapcenter(OSM_GPS_MAP(context->widget), 
 			    context->appdata->map.pos.lat, 
 			    context->appdata->map.pos.lon, 
@@ -1045,7 +1145,6 @@ void map(appdata_t *appdata) {
 
   /* install handler for timed updates of the gps button */
   context->handler_id = gtk_timeout_add(1000, map_gps_update, context);
-  context->cb_id = gps_register_callback(appdata, gps_callback, context);
 
 #if MAEMO_VERSION_MAJOR == 5
   /* prevent some of the main screen things */
@@ -1056,6 +1155,11 @@ void map(appdata_t *appdata) {
   g_signal_connect(G_OBJECT(context->window), "destroy", 
 		   G_CALLBACK(on_window_destroy), context);
 
+  /* connect a key handler to forward global shortcuts (function keys) */
+  /* to the ap widget */
+  g_signal_connect(G_OBJECT(context->window), "key_press_event",
+		   G_CALLBACK(on_map_window_key_press), context->widget);
+
   gtk_container_add(GTK_CONTAINER(context->window), context->widget); 
   gtk_widget_show_all(GTK_WIDGET(context->window));
 
