@@ -33,7 +33,6 @@ typedef struct map_context_s {
   gint handler_id;
   cache_t *balloon, *press_on;
   map_state_t state;
-  void *cb_id;
 
   gboolean goto_is_enabled;
 
@@ -44,8 +43,10 @@ typedef struct map_context_s {
   GtkWidget *old_view;
 #endif
 
-  /* save various values defining the overlay state */
-  /* in order to repaint the overlays only if necessary */
+  struct {
+    gps_mask_t set;
+    struct gps_t fix;
+  } gps;
 
 } map_context_t;
 
