@@ -131,4 +131,11 @@ GtkWidget *export_file(char *sel_title, char **filename);
 GtkWidget *number_editor_new(int current, int min, int max);
 int number_editor_get_value(GtkWidget *widget);
 
+GtkWidget *notebook_new(void);
+void notebook_append_page(GtkWidget *notebook, GtkWidget *page, char *label);
+GtkWidget *notebook_get_gtk_notebook(GtkWidget *notebook);
+
+GtkWidget *text_wrap(char *str);
+void text_set(GtkWidget *view, char *str);
+
 #endif // MISC_H
